let svg = d3.select('svg.center');

const row = '#236297',
      arr = '#e4f4fb',
      plasma_fn = (i,f) => f(d3.hsl(d3.interpolatePlasma(0.5+0.2*i))),
      plasma = (i,f) => plasma_fn(i, c=>c.copy({l:f(c.l)})),
      red = plasma(1, l=>0.85*l);

svg.append('defs').append('marker').attrs({
        viewBox:'0 0 7 5', markerWidth:8, markerHeight:8,
        orient:'auto', refX:3, refY:2.5, fill:'none', stroke:red, id:'arrow',
        'stroke-width':0.75
    })
    .append('path').attr('d','M0 0L3.5 2.5L0 5');

const W = 370,
      stagger = 50;
let g = svg.append('g').attr('opacity',1);
g.selectAll().data(d3.range(3)).enter().append('g')
    .attr('transform', i=>`translate(${600+W*(i-1.1)},${stagger*i})`)
    .call(a => {
        const w = 300,
              y0= 100,
              h = 250,
              sh= [5,6],
              b = sh[0]*sh[1],
              x = j => -w/2+(j+1/2)*(w/sh[1]),
              y = i => y0+(i+1/2)*h/sh[0];
        a.selectAll().data(d3.range(sh[0]))
            .join('rect').attrs(i => ({
                x:-w/2, width:w, y:y(i-1/2), height:h/sh[0],
                'stroke-width':2, stroke:row, fill:arr
            }));
        a.selectAll().data(d3.range(sh[1]))
            .join('line').attrs({
                x1:x, x2:x,
                y1:y0+h-10, y2:y0+16,
                'stroke-width':5, stroke:red, 'marker-end':'url(#arrow)',
                opacity:0.86
            });
        a.selectAll().data(k => d3.range(b).map(n=>k*b+n))
            .join('text').attrs(n => ({
                x:x(n%sh[1]), y:y(Math.floor(n/sh[1])%sh[0]),
                'text-anchor':'middle', 'font-size':26, 'dy':'0.32em',
                fill:'black', opacity:0.8
            }))
            .text(n=>1+n);

        let label = sel => {
            let vadd = (a,b)=>a.map((e,i)=>e+b[i]);
            let mp = (m,d) => {
                let a = Math.sqrt(d.reduce((a,b)=>a*a+b*b)),
                    s = d.map(e=>e/a),
                    o = s.slice().reverse(); o[1]*=-1,
                    v = [d,o,s];
                return m.map(r=>v.map((e,i)=>e.map(f=>r[i]*f)).reduce(vadd));
            }
            let color = pd => plasma(pd[3], l=>l/1.5);
            sel.append('path').attrs(pd => ({
                    d:'M'+[pd[0]].concat(mp([[0,15,2],[1,0,-4],[0,-15,2]], pd[1]))
                        .map(e=>e.join(' ')).join('l'),
                    'stroke-width':4, stroke:color, fill:'none'
                }));
            sel.append('text').attrs(pd => {
                    let d = pd[1],
                        p = vadd(mp([[1/2,42,0]],d)[0], pd[0]),
                        r = Math.atan2(d[1],Math.abs(d[0]))/(2*Math.PI);
                    return {
                        transform:`translate(${p.join(',')})rotate(${360*r})`,
                        'font-size':40, fill:'#466',
                        fill:color, dy:'0.36em', 'text-anchor':'middle'
                    };
                })
                .text(pd=>pd[2]);
        }
        a.selectAll().data(i =>
                i===0?[ [[x(sh[1]-1/2),y(-2/3)],[2*W,2*stagger],'LEADING', 0] ]
              : i===2?[ [[x(sh[1]-1/3),y(-1/2)], [0,h], '←REDUCTION', 1],
                        [[x(sh[1]-1/2),y(sh[0]-1/3)], [-w,0], 'TRAILING', 2] ]
              : []
            ).enter().call(label);
    });

const w = 120,
      h = 250,
      ty = 96;

function getdim(shape, axis) {
    let rank=shape.length, dim=[1, 1, 1], i;
    if (shape.length) {
        for (i=0; i<axis; i++) dim[0] *= shape[i];
        dim[1] = shape[i++];
        for (   ; i<rank; i++) dim[2] *= shape[i];
    }
    return dim;
}

let backColor = (_,i) => plasma(i, l=>1-(1-l)/2.8);

function draw(shape, axis) {
    let rank = shape.length,
        dim  = getdim(shape, axis),
        cent = (shape.length-1)/2;

    let part = [0,axis,Math.min(axis+1,rank),rank],
        g = 8/w;
    if (part[1]===part[2]) { part[1]-=g/2; part[2]+=g/2; }
    part[0] = Math.min(part[0],part[1]-g);
    part[3] = Math.max(part[3],part[2]+g);

    function path(a, b, up) {
        let x0 = (a-cent-1/2)*w,
            x1 = (b-3/2)*w,
            xd = x1-x0,
            l  = 120,
            yd = h-l,
            c  = 40;
        return (up ? 'L'+x1+' '+(h+l) : 'M'+x0+' 0')
          + ['v',l,'c',0,c,xd,(yd-c),xd,yd,'v',l]
                .map(x=>isNaN(x)?x:up?-x:x)
                .join(' ');
    }

    let grp = svg.append('g').attr('opacity',0)
        .attrs({'text-anchor':'middle',
                'font-family':'monospace',
                'font-size':100,
                'transform':'translate(600,100)'});

    grp.selectAll().data([0,1,2].map(i => [part[i],part[i+1]])).enter()
        .append('path')
        .attr('fill', backColor)
        .attr('stoke', 'none')
        .attr('d', (p,i) => p.map((l,j)=>path(l,i+j,j)).join('')+'Z');

    grp.selectAll().data([0,2].map(i => ((part[i]+part[i+1]-1)/2-cent + 2*(i-1))/3)).enter()
        .append('text').text('×')
        .attrs({'font-size':72, fill:'#446'})
        .attr('x', x=>x*w)
        .attr('y', ty+0.6*h-16);

    grp.selectAll().data(part).enter()
        .append('path')
        .attrs({fill:'none', 'stroke-width':4, stroke:'#334'})
        .attr('d', (a,b)=>path(a,b,false));

    grp.selectAll().data(shape).enter()
        .append('text').text(a=>a)
        .attr('x', (_,i)=>(i-cent)*w)
        .attr('y', ty);
    grp.selectAll().data(dim).enter()
        .append('text').text(a=>a)
        .attr('x', (_,i)=>(i-1)*w)
        .attr('y', ty+h);

   return grp;
}

let axes = [
    g,
    draw([4,2,5,7], 2),
    draw([4,2,5,7], 0),
    draw([58], 0),
    draw([2,3,2], 2),
];
let step = 0;
document.onclick = function () {
    if (step === axes.length-1) return;
    axes[step].transition().duration(800).attr('opacity',0);
    axes[++step].transition().delay(400).duration(1200).attr('opacity',1);
}
